// Jeremy Stribling, 13794235
// cs184-cf, sec. 102
// 10/2/00
// PA #3

// This code simulates rotating a cube in 3D

#include "stdafx.h"
#include <math.h>
#include <stdlib.h>
#include "f:\cs184\glut-3.7\glut.h"
#include <gl\gl.h>

// a struct representing a color RGB

typedef struct colorStruct {
	double red;
	double green;
	double blue;
} Color;

// a struct representing a vector in 3D

typedef struct vectorStruct {
	double x;
	double y;
	double z;
} Vector;

// a struct representing a 2D point to be rendered

typedef struct point2dStruct {
	double x;
	double y;
} Point2d;

// a sruct representing a 3d rotation, with an axis and an angle

typedef struct rotationStruct {

	Vector * axis;
	double angle;

} Rotation;


// a struct representing a frame of animation

typedef struct frameStruct {
	Vector * vertices[8];
	Vector * center;
	Vector * translate;
	Rotation * rotation;
} Frame;

// Static variables representing whether a mouse button is being pressed or not

int PRESSED = 1;
int NOT_PRESSED = 0;

// Variables representing whther the left/right buttons are being pressed

int mouseLeft = NOT_PRESSED;
int mouseRight = NOT_PRESSED;

// The current height and width of the window

double windowHeight;
double windowWidth;

// doubles telling where the place the mouse was during a button press

double lastX = -1;
double lastY = -1;

// The length of the side of the image plane in world coordinates

int SIZE_OF_IMAGE_PLANE_X = 3;
int SIZE_OF_IMAGE_PLANE_Y = 3;

// a vector defining where the camera is in world coordinates

Vector * camera;

// a Vector defining the center of the cube in 3D.

Vector * center;
Vector * absoluteCenter;

// The 8 vertices that define the cube in world coordinates

Vector * cubeVertices[ 8 ];

// set of frames

Frame ** frames;

// The current rotation from the original position to now

Rotation * currRotation;

// colors to define

Color * BLACK;
Color * WHITE;
Color * PINK;
Color * RED;
Color * LIGHT_GREEN;
Color * GREEN;
Color * LIGHT_BLUE;
Color * BLUE;
Color * GOLD;
Color * YELLOW;
Color * VIOLET;
Color * MAGENTA;
Color * SEA_GREEN;
Color * TURQUOISE;

// Vector constuctor

Vector * makeVector( double x, double y, double z ) {

	Vector * newVect = (Vector *) malloc( sizeof( Vector ) );
	newVect->x = x;
	newVect->y = y;
	newVect->z = z;
	return newVect;

}

Rotation * makeRotation( Vector * ax, double an ) {

	Rotation * newr = (Rotation *) malloc( sizeof( Rotation ) );
	newr->axis = ax;
	newr->angle = an;
	return newr;

}

// Frame Constructor

Frame * makeFrame() {

	Frame * newFrame = (Frame *) malloc( sizeof(Frame) );

	newFrame->vertices[0] = makeVector( 1.0, 1.0, 3.0 );
	newFrame->vertices[1] = makeVector( -1.0, 1.0, 3.0 );
	newFrame->vertices[2] = makeVector( -1.0, -1.0, 3.0 );
	newFrame->vertices[3] = makeVector( 1.0, -1.0, 3.0 );
	newFrame->vertices[4] = makeVector( 1.0, 1.0, 5.0 );
	newFrame->vertices[5] = makeVector( -1.0, 1.0, 5.0 );
	newFrame->vertices[6] = makeVector( -1.0, -1.0, 5.0 );
	newFrame->vertices[7] = makeVector( 1.0, -1.0, 5.0 );

	newFrame->center = makeVector( 0.0, 0.0, 4.0 );

	newFrame->translate = makeVector( 0.0, 0.0, 0.0 );

	newFrame->rotation = makeRotation( makeVector( 1.0, 0.0, 0.0 ), 0.0 );

	return newFrame;

}

void freeFrame( Frame * f ) {

	free( f->center );
	free( f->translate );
	for( int i = 0; i < 8; i++ ) {
		free( f->vertices[i] );
	}
	free( f->rotation->axis );
	free( f );
}

// a 2D Point constructor

Point2d * makePoint2d( double x, double y ) {

	Point2d * newPoint = (Point2d *) malloc( sizeof( Point2d ) );
	newPoint->x = x;
	newPoint->y = y;
	return newPoint;

}

// Create a color struct, given the R, G, and B values

Color * makeColor( double r, double g, double b) {

	Color * theColor = (Color *) malloc (sizeof(Color));
	theColor->blue = b;
	theColor->green = g;
	theColor->red = r;
	return theColor;

}

// Change the color OpenGL is  drawing with

void changeColor( Color * theColor ) {

	glColor3f( theColor->red, theColor->green, theColor->blue );

}

// returns a new vector representing the difference between the two given vectors.

Vector * subVectors( Vector * vect1, Vector * vect2 ) {

	double x = vect1->x - vect2->x;
	double y = vect1->y - vect2->y;
	double z = vect1->z - vect2->z;
	return makeVector( x, y, z );

}

// Computes the dot product between two vectors

double dotProduct( Vector * vect1, Vector * vect2 ) {

	return (vect1->x * vect2->x) + (vect1->y * vect2->y) + (vect1->z * vect2->z);

}

// Compute cross product between two vectors

Vector * crossProduct( Vector * vect1, Vector * vect2 ) {

	double x = ( vect1->y * vect2->z ) - ( vect1->z * vect2->y );
	double y = ( vect1->z * vect2->x ) - ( vect1->x * vect2->z );
	double z = ( vect1->x * vect2->y ) - ( vect1->y * vect2->x );
	return makeVector( x, y, z);

}

// Returns the length of the given axis

double getLength( Vector * vect ) {

	return sqrt( (vect->x)*(vect->x) + (vect->y)*(vect->y) + (vect->z)*(vect->z) );

}

/////////////  The following four functions were originally written by Gavin Bell   ////////////////////////////
/////////////  I found it free on the web at http://www.ddnet.es/personales/paulo/vrml/rotat.phtml, /////////////
/////////////  and modified it according to my needs ////////////////////////////////////////////////////////////

/*
 * Convert axis/angle to quaternion form:
 * Assumes axis is unit-length, angle is in radians.
 */
void
to_quaternion(float axis[3], float angle, float result[4])
{
    int i;
    for (i = 0; i < 3; i++) {
		result[i] = axis[i]*sin(angle / 2.0);
    }
    result[3] = cos(angle / 2.0);
}
/*
 * Convert unit-length quaternion to axis/angle.
 */
void
from_quaternion(float q[4], float axis[3], float *angle)
{
    int i;
    *angle = acos(q[3]) * 2.0;
    for (i = 0; i < 3; i++) {
		axis[i] = q[i]/ sin(*angle / 2 );
    }
}

/*
 * Multiply two quaternions together.  Result may be the same as
 * either input:
 */
void
multiply_quaternion(float q1[4], float q2[4], float result[4])
{
    float tmp[4];
    tmp[0] = q2[3] * q1[0] + q2[0] * q1[3] - q2[1] * q1[2] + q2[2] * q1[1];
    tmp[1] = q2[3] * q1[1] + q2[1] * q1[3] - q2[2] * q1[0] + q2[0] * q1[2];
    tmp[2] = q2[3] * q1[2] + q2[2] * q1[3] - q2[0] * q1[1] + q2[1] * q1[0];
    tmp[3] = q2[3] * q1[3] - q2[0] * q1[0] - q2[1] * q1[1] - q2[2] * q1[2];
    result[0] = tmp[0]; result[1] = tmp[1];
    result[2] = tmp[2]; result[3] = tmp[3];
}

Rotation * addRotations( float axis[2][3], float angle[2] ) {

	float q1[4], q2[4], result_axis[3], result_angle;

	to_quaternion(axis[0], angle[0], q1);
	to_quaternion(axis[1], angle[1], q2);

	multiply_quaternion( q1, q2, q1 );

    from_quaternion(q1, result_axis, &result_angle);

	return makeRotation( makeVector( result_axis[0], result_axis[1], result_axis[2] ), result_angle );
}

///////////////////////////////////////  End of Bell code ////////////////////////////////

///////////////////I found the following function in its original form for free on the web at/////////////
//////////////////http://www.gamasutra.com/features/programming/19980703/quaternions_01.htm///////////////
//////////////////  I modified it to suit my own purposes  ////////////////////////////////////////////////

void quatSlerp(float from[4], float to[4], float t, float res[4])
      {
        float           to1[4];
        double        omega, cosom, sinom, scale0, scale1;

		float DELTA = .0001;

        // calc cosine
        cosom = from[0] * to[0] + from[1] * to[1] + from[2] * to[2]
			       + from[3] * to[3];

        // adjust signs (if necessary)
        if ( cosom <0.0 ){ 
			cosom = -cosom; 
			to1[0] = - to[0];
			to1[1] = - to[1];
			to1[2] = - to[2];
			to1[3] = - to[3];
        } else  {
			to1[0] = to[0];
			to1[1] = to[1];
			to1[2] = to[2];
			to1[3] = to[3];
        }

        // calculate coefficients

       if ( (1.0 - cosom) > DELTA ) {
                // standard case (slerp)
                omega = acos(cosom);
                sinom = sin(omega);
                scale0 = sin((1.0 - t) * omega) / sinom;
                scale1 = sin(t * omega) / sinom;

        } else {        
    // "from" and "to" quaternions are very close 
	    //  ... so we can do a linear interpolation
                scale0 = 1.0 - t;
                scale1 = t;
        }
	// calculate final values
	res[0] = scale0 * from[0] + scale1 * to1[0];
	res[1] = scale0 * from[1] + scale1 * to1[1];
	res[2] = scale0 * from[2] + scale1 * to1[2];
	res[3] = scale0 * from[3] + scale1 * to1[3];
}

/////////////////////////////////////////////////////////////////////////

void makeFrames( int start, int end ) {

	for( int i = start; i <= end; i++ ) {

		frames[i] = makeFrame();

	}

}

void changeNumFrames( int numFrames, int deltaFrames ) {

	Frame ** frTemp = new Frame * [ numFrames + deltaFrames ];
	if( deltaFrames < 0 ) {

		for( int i = 0; i < (numFrames + deltaFrames); i++ ) {
			frTemp[i] = frames[i];
		}
		for( i = (numFrames + deltaFrames); i < numFrames; i++ ) {
			freeFrame( frames[i] );
		}

	} else {

		for( int i = 0; i < numFrames; i++ ) {
			frTemp[i] = frames[i];
		}
		for( i = numFrames; i < (numFrames + deltaFrames); i++ ) {
			frTemp[i] = makeFrame();
		}
	}
	frames = frTemp;

}

void storeFrame( int f ) {

	for( int i = 0; i < 8; i++ ) {
		frames[f]->vertices[i]->x = cubeVertices[i]->x;
		frames[f]->vertices[i]->y = cubeVertices[i]->y;
		frames[f]->vertices[i]->z = cubeVertices[i]->z;
	}
	frames[f]->center->x = center->x;
	frames[f]->center->y = center->y;
	frames[f]->center->z = center->z;
	frames[f]->translate->x = (center->x - absoluteCenter->x );
	frames[f]->translate->y = (center->y - absoluteCenter->y );
	frames[f]->translate->z = (center->z - absoluteCenter->z );

	frames[f]->rotation->axis->x = currRotation->axis->x;
	frames[f]->rotation->axis->y = currRotation->axis->y;
	frames[f]->rotation->axis->z = currRotation->axis->z;
	frames[f]->rotation->angle = currRotation->angle;

}

void loadFrame( int f ) {

	for( int i = 0; i < 8; i++ ) {
		cubeVertices[i]->x = frames[f]->vertices[i]->x;
		cubeVertices[i]->y = frames[f]->vertices[i]->y;
		cubeVertices[i]->z = frames[f]->vertices[i]->z;
	}
	center->x = frames[f]->center->x;
	center->y = frames[f]->center->y;
	center->z = frames[f]->center->z;

	currRotation->axis->x = frames[f]->rotation->axis->x;
	currRotation->axis->y = frames[f]->rotation->axis->y;
	currRotation->axis->z = frames[f]->rotation->axis->z;
	currRotation->angle = frames[f]->rotation->angle;
}

// Returns the normalized version of the given vector

Vector * getUnitAxis( Vector * vect ) {

	double length = getLength( vect );
	return makeVector( (vect->x)/length,(vect->y)/length, (vect->z)/length );
	

}

// translate cube by the given z value

void translate( Vector * amount ) {

	int i;

	for( i = 0; i < 8; i++ ) {
		(cubeVertices[i])->x = (cubeVertices[i])->x + amount->x;
		(cubeVertices[i])->y = (cubeVertices[i])->y + amount->y;
		(cubeVertices[i])->z = (cubeVertices[i])->z + amount->z;
	}

	center->x = center->x + amount->x;
	center->y = center->y + amount->y;
	center->z = center->z + amount->z;

}

// Find the matrix that rotates the given unit axis to the z axis

Vector ** getRotateToZ( Vector * unitAxis ) {

	double a = unitAxis->x;
	double b = unitAxis->y;
	double c = unitAxis->z;
	double d = sqrt( b*b + c*c );
	Vector * topRow;
	Vector * middleRow;
	Vector * bottomRow;
	
	if( d == 0.0 ) {
		topRow = makeVector( d, 0, -a );
		middleRow = makeVector( 0, 1, 0 );
		bottomRow = makeVector( a, 0, d );
	} else {
		topRow = makeVector( d, -(a*b)/d, -(a*c)/d );
		middleRow = makeVector( 0, c/d, -b/d );
		bottomRow = makeVector( a, b, c );
	}

	Vector ** matrix = new Vector * [3];
	matrix[0] = topRow;
	matrix[1] = middleRow;
	matrix[2] = bottomRow;

	return matrix;

}

// Rotates the cube by the given rotation matrix

void performRotation( Vector * matrix[] ) {

	int i;

	for( i = 0; i < 8; i++ ) {

		double newX = dotProduct( matrix[0], cubeVertices[i] );
		double newY = dotProduct( matrix[1], cubeVertices[i] );
		double newZ = dotProduct( matrix[2], cubeVertices[i] );
		(cubeVertices[i])->x = newX;
		(cubeVertices[i])->y = newY;
		(cubeVertices[i])->z = newZ;
	}

}

// Find the matrix that rotate the cube, which should now have it's 
// axis of rotation on the z axis, through the given angle

Vector ** getRotateThroughAngle( double angle ) {

	Vector * topRow = makeVector( cos( angle ), sin( angle ), 0 );
	Vector * middleRow = makeVector( -sin( angle ), cos( angle ), 0 );
	Vector * bottomRow = makeVector( 0, 0, 1 );

	Vector ** matrix = new Vector * [3];
	matrix[0] = topRow;
	matrix[1] = middleRow;
	matrix[2] = bottomRow;

	return matrix;

}

// Finds the transpose of the given matrix. Since we are working with 
// rotation matrices, this also returns the inverse.

Vector ** transpose( Vector * matrix[] ) {

	Vector * topRow = makeVector( (matrix[0])->x, (matrix[1])->x, (matrix[2])->x );
	Vector * middleRow = makeVector( (matrix[0])->y, (matrix[1])->y, (matrix[2])->y );
	Vector * bottomRow = makeVector( (matrix[0])->z, (matrix[1])->z, (matrix[2])->z );

	Vector ** tMatrix = new Vector * [3];
	tMatrix[0] = topRow;
	tMatrix[1] = middleRow;
	tMatrix[2] = bottomRow;

	return tMatrix;

}

// Free the memory used by the Vectors in this matrix

void freeMatrix( Vector * matrix[] ) {

	free( matrix[0] );
	free( matrix[1] );
	free( matrix[2] );
	free( matrix );

}

// Rotate the cube around the given vector by the specified angle.

void rotateCube( Vector * axis, double angle ) {
	
	if( angle == 0 ) {
		return;
	}

	Vector * unitAxis = getUnitAxis( axis );
	Vector ** rotateToZ;
	Vector ** rotateFromZ;
	Vector ** rotateThroughAngle;

	Vector * amount = makeVector( -center->x, -center->y, -center->z );

	float axisAr[2][3];
	float angleAr[2];

	axisAr[0][0] = currRotation->axis->x;
	axisAr[0][1] = currRotation->axis->y;
	axisAr[0][2] = currRotation->axis->z;
	axisAr[1][0] = unitAxis->x;
	axisAr[1][1] = unitAxis->y;
	axisAr[1][2] = unitAxis->z;
	angleAr[0] = currRotation->angle;
	angleAr[1] = angle;

	free( currRotation->axis );

	currRotation = addRotations( axisAr, angleAr );

	translate( amount );
	rotateToZ = getRotateToZ( unitAxis );
	performRotation( rotateToZ );
	rotateThroughAngle = getRotateThroughAngle( angle );
	performRotation( rotateThroughAngle );
	rotateFromZ = transpose( rotateToZ );
	performRotation( rotateFromZ );

	amount->x = -amount->x;
	amount->y = -amount->y;
	amount->z = -amount->z;

	translate( amount );

	free( amount );
	free( unitAxis );
	freeMatrix( rotateToZ );
	freeMatrix( rotateThroughAngle );
	freeMatrix( rotateFromZ );

}

// Converts a vector in 3D into a point which can be rendered.

Point2d * convertVectToPoint( Vector * vect ) {

	double pointX;
	double pointY;
	double vectX = vect->x;
	double vectY = vect->y;
	double vectZ = vect->z;

	pointX = vectX/( vectZ * SIZE_OF_IMAGE_PLANE_X) + 1.0/2.0;
	pointY = vectY/( vectZ * SIZE_OF_IMAGE_PLANE_Y) + 1.0/2.0;

	return makePoint2d( pointX, pointY );

}

// Decides whether or not the plane represented by it's normal 
//vector and a point on it is visible by the camera.

int isVisible( Vector * normal, Vector * point ) {

	Vector * viewVector = subVectors( point, camera );

	return dotProduct( viewVector, normal ) <= 0;

}

// Returns an angle through which to rotate the cube.
// Returns 2*pi if the length of the segment represented by
// the two given doubles is the same as the diagonal of the window,
// and scales the angle down according to the how much shorter
// the length is than the diagonal.

double getAngle( double diffX, double diffY ) {

	double length = sqrt( diffX*diffX + diffY*diffY );

	double angle = 2*3.14159*length/(sqrt(2));

	return angle;

}

void frontAndCenter() {

	cubeVertices[0] = makeVector( 1.0, 1.0, 3.0 );
	cubeVertices[1] = makeVector( -1.0, 1.0, 3.0 );
	cubeVertices[2] = makeVector( -1.0, -1.0, 3.0 );
	cubeVertices[3] = makeVector( 1.0, -1.0, 3.0 );
	cubeVertices[4] = makeVector( 1.0, 1.0, 5.0 );
	cubeVertices[5] = makeVector( -1.0, 1.0, 5.0 );
	cubeVertices[6] = makeVector( -1.0, -1.0, 5.0 );
	cubeVertices[7] = makeVector( 1.0, -1.0, 5.0 );
	
	center = makeVector( 0.0, 0.0, 4.0 );

	currRotation = makeRotation ( makeVector( 0.0, 0.0, 0.0), 0.0 );

}

void interpolate( int from, int to ) {

	int frameDiff = to - from;
	if( frameDiff <= 0 ) {
		return;
	}

	Frame * f1 = frames[from];
	Frame * f2 = frames[to];

	/*Vector * diff0 = subVectors( f2->vertices[0], f1->vertices[0] );
	Vector * diff1 = subVectors( f2->vertices[1], f1->vertices[1] );
	Vector * diff2 = subVectors( f2->vertices[2], f1->vertices[2] );
	Vector * diff3 = subVectors( f2->vertices[3], f1->vertices[3] );
	Vector * diff4 = subVectors( f2->vertices[4], f1->vertices[4] );
	Vector * diff5 = subVectors( f2->vertices[5], f1->vertices[5] );
	Vector * diff6 = subVectors( f2->vertices[6], f1->vertices[6] );
	Vector * diff7 = subVectors( f2->vertices[7], f1->vertices[7] );*/
	Vector * centerDiff = subVectors( f2->center, f1->center );

	//Vector * axis = crossProduct( vect2, vect1 );
	
	//double angle = acos( dotProduct( vect1, vect2 )/( getLength(vect1)*getLength(vect2) ) );	
/*
	CString a;
	a.Format( "%f, %f=%fX%f, %f=%fX%f, %f=%fX%f", angle*360/(2*3.14159), axis->x, vect2->x, vect1->x, axis->y, vect2->y, vect1->y, axis->z, vect2->z, vect1->z );
	AfxMessageBox( a );
*/
	Frame * currFrame;
	double factor = 1/(frameDiff*1.0);

	// conert rotations to quaternions

	float rot1[3];
	float rot2[3];
	float q1[4];
	float q2[4];
	float resultQ[4];
	float resultR[3];
	float angle;
	rot1[0] = f1->rotation->axis->x;
	rot1[1] = f1->rotation->axis->y;
	rot1[2] = f1->rotation->axis->z;
	rot2[0] = f2->rotation->axis->x;
	rot2[1] = f2->rotation->axis->y;
	rot2[2] = f2->rotation->axis->z;

	to_quaternion( rot1, f1->rotation->angle, q1 );
	to_quaternion( rot2, f2->rotation->angle, q2 );

	Vector * axis = makeVector( 0, 0, 0 );

	for( int i=1; i < frameDiff; i++ ) {

		loadFrame( from );

		translate( makeVector( -center->x, -center->y, -center->z ) );

		currFrame = frames[from + i];

		currFrame->center->x = f1->center->x + (centerDiff->x)*factor*i;
		currFrame->center->y = f1->center->y + (centerDiff->y)*factor*i;
		currFrame->center->z = f1->center->z + (centerDiff->z)*factor*i;
		currFrame->translate->x = (currFrame->center->x - absoluteCenter->x );
		currFrame->translate->y = (currFrame->center->y - absoluteCenter->y );
		currFrame->translate->z = (currFrame->center->z - absoluteCenter->z );

		quatSlerp( q1, q2, factor*i, resultQ );
	
		from_quaternion( resultQ, resultR, &angle );

		axis->x = resultR[0];
		axis->y = resultR[1];
		axis->z = resultR[2];

		frontAndCenter();

		translate( makeVector( -center->x, -center->y, -center->z ) );

		rotateCube( axis, angle );
		
		for( int j = 0; j < 8; j++ ) {

			currFrame->vertices[j]->x = cubeVertices[j]->x + currFrame->center->x;
			currFrame->vertices[j]->y = cubeVertices[j]->y + currFrame->center->y;
			currFrame->vertices[j]->z = cubeVertices[j]->z + currFrame->center->z;

		} 

		currFrame->rotation->axis->x = currRotation->axis->x;
		currFrame->rotation->axis->y = currRotation->axis->y;
		currFrame->rotation->axis->z = currRotation->axis->z;
		currFrame->rotation->angle = currRotation->angle;

		/*currFrame->vertices[0]->x = f1->vertices[0]->x + (diff0->x)*factor;
		currFrame->vertices[0]->y = f1->vertices[0]->y + (diff0->y)*factor;
		currFrame->vertices[0]->z = f1->vertices[0]->z + (diff0->z)*factor;
		currFrame->vertices[1]->x = f1->vertices[1]->x + (diff1->x)*factor;
		currFrame->vertices[1]->y = f1->vertices[1]->y + (diff1->y)*factor;
		currFrame->vertices[1]->z = f1->vertices[1]->z + (diff1->z)*factor;
		currFrame->vertices[2]->x = f1->vertices[2]->x + (diff2->x)*factor;
		currFrame->vertices[2]->y = f1->vertices[2]->y + (diff2->y)*factor;
		currFrame->vertices[2]->z = f1->vertices[2]->z + (diff2->z)*factor;
		currFrame->vertices[3]->x = f1->vertices[3]->x + (diff3->x)*factor;
		currFrame->vertices[3]->y = f1->vertices[3]->y + (diff3->y)*factor;
		currFrame->vertices[3]->z = f1->vertices[3]->z + (diff3->z)*factor;
		currFrame->vertices[4]->x = f1->vertices[4]->x + (diff4->x)*factor;
		currFrame->vertices[4]->y = f1->vertices[4]->y + (diff4->y)*factor;
		currFrame->vertices[4]->z = f1->vertices[4]->z + (diff4->z)*factor;
		currFrame->vertices[5]->x = f1->vertices[5]->x + (diff5->x)*factor;
		currFrame->vertices[5]->y = f1->vertices[5]->y + (diff5->y)*factor;
		currFrame->vertices[5]->z = f1->vertices[5]->z + (diff5->z)*factor;
		currFrame->vertices[6]->x = f1->vertices[6]->x + (diff6->x)*factor;
		currFrame->vertices[6]->y = f1->vertices[6]->y + (diff6->y)*factor;
		currFrame->vertices[6]->z = f1->vertices[6]->z + (diff6->z)*factor;
		currFrame->vertices[7]->x = f1->vertices[7]->x + (diff7->x)*factor;
		currFrame->vertices[7]->y = f1->vertices[7]->y + (diff7->y)*factor;
		currFrame->vertices[7]->z = f1->vertices[7]->z + (diff7->z)*factor;*/

	}

	//free( vect1 );
	//free( vect2 );
	free( axis );
	free( centerDiff );

}

// The display function. It is called whenever the window needs
// redrawing (ie: overlapping window moves, resize, maximize)
// You should redraw your polygons here
void display(void) {

	int i;
	Vector * normal;

	// Clear the background
	glClear(GL_COLOR_BUFFER_BIT);

	// Draw each face of the cube, if visible

	for( i = 1; i <= 6; i++ ) {
			
		Vector * point1;
		Vector * point2;
		Vector * point3;
		Vector * point4;
		Color * edgeColor;
		//Vector * normal;

		// figure out which points and colors to use for the edges

		switch( i ) {
		case 1:
			point1 = cubeVertices[ 0 ];
			point2 = cubeVertices[ 3 ];
			point3 = cubeVertices[ 2 ];
			point4 = cubeVertices[ 1 ];
			edgeColor = RED;
			break;
		case 2:
			point1 = cubeVertices[ 1 ];
			point2 = cubeVertices[ 2 ];
			point3 = cubeVertices[ 6 ];
			point4 = cubeVertices[ 5 ];
			edgeColor = GREEN;
			break;
		case 3:
			point1 = cubeVertices[ 5 ];
			point2 = cubeVertices[ 6 ];
			point3 = cubeVertices[ 7 ];
			point4 = cubeVertices[ 4 ];
			edgeColor = BLUE;
			break;
		case 4:
			point1 = cubeVertices[ 4 ];
			point2 = cubeVertices[ 7 ];
			point3 = cubeVertices[ 3 ];
			point4 = cubeVertices[ 0 ];
			edgeColor = YELLOW;
			break;
		case 5:
			point1 = cubeVertices[ 0 ];
			point2 = cubeVertices[ 1 ];
			point3 = cubeVertices[ 5 ];
			point4 = cubeVertices[ 4 ];
			edgeColor = MAGENTA;
			break;
		default:
			point1 = cubeVertices[ 3 ];
			point2 = cubeVertices[ 7 ];
			point3 = cubeVertices[ 6 ];
			point4 = cubeVertices[ 2 ];
			edgeColor = TURQUOISE;
		}

		normal = crossProduct( subVectors( point2, point1 ), subVectors( point4, point1 ) );

		if( isVisible( normal, point1 ) ) {

			Point2d * topLeft = convertVectToPoint( point1 );
			Point2d * topRight = convertVectToPoint( point4 );
			Point2d * bottomLeft = convertVectToPoint( point2 );
			Point2d * bottomRight = convertVectToPoint( point3 );
			
			changeColor( edgeColor );
			
			glBegin( GL_POLYGON );
			glVertex2f( topLeft->x, topLeft->y );
			glVertex2f( topRight->x, topRight->y );
			glVertex2f( bottomRight->x, bottomRight->y );
			glVertex2f( bottomLeft->x, bottomLeft->y );
			glEnd();

		}

	}

	glutSwapBuffers();

	// (Note that the origin is lower left corner)
	// (Note also that the window spans (0,1) )
	// Finish drawing, update the frame buffer

	glFinish();

}

// This function is called whenever the window is resized. 
// Parameters are the new dimentions of the window
void	resize(int x,int y)
{
	windowHeight = glutGet( GLUT_WINDOW_HEIGHT )*1.0;
	windowWidth = glutGet( GLUT_WINDOW_WIDTH )*1.0;
	glViewport(0,0,x,y);

}

// This functions is called whenever the mouse is pressed or released
// button is a number 0 to 2 designating the button
// state is 1 for release 0 for press event
void	mouse(int button,int state,int x,int y)
{
	double normalX = x/windowWidth;
	double normalY = -(y - windowHeight)/windowHeight;
	
	if( button == 0 ) {
		if( state == 0 ) {
			mouseLeft = PRESSED;
			lastX = normalX;
			lastY = normalY;
		} else {
			mouseLeft = NOT_PRESSED;
			lastX = -1;
			lastY = -1;
		}
	} else if( button == 2 ) {
		if( state == 0 ) {
			mouseRight = PRESSED;
			lastX = normalX;
			lastY = normalY;
		} else {
			mouseRight = NOT_PRESSED;
			lastX = -1;
			lastY = -1;
		}
	} 

}

void mouseMoveXY( double diffX, double diffY ) {

	Vector * amount = makeVector( diffX, diffY, 0.0 );

	translate( amount );

	free( amount );

}

void mouseMoveZ( double diff ) {

	Vector * amount = makeVector( 0.0, 0.0, diff );

	translate( amount );

	free( amount );

}


// called every time the mouse moves with a button pressed.
// computes about what axis and through what angle the cube
// should rotate.


void mouseMove( int x, int y ) {

	double normalX = x/windowWidth;
	double normalY = -(y - windowHeight)/windowHeight;
	double diffX;
	double diffY;
	Vector * axis;
	double angle;
	double scale = .25;

	if( mouseRight == PRESSED && mouseLeft == PRESSED ) {

		if( lastX == -1 ) {
			lastX = normalX;
			lastY = normalY;
			return;
		}

		diffY = (normalY - lastY)/scale;
	
		mouseMoveZ( diffY );

		lastX = normalX;
		lastY = normalY;

	} else if( mouseRight == PRESSED ) {	
		
		// Rotate the cube around an axis perpendicular to the mouse stroke

		if( lastX == -1 ) {
			lastX = normalX;
			lastY = normalY;
			return;
		}

		diffX = normalX - lastX;
		diffY = normalY - lastY;
		
		if( diffX == 0 && diffY == 0 ) {
			return;
		}
		
		angle = getAngle( diffX, diffY );
		axis = makeVector( -diffY, diffX, 0 );
		rotateCube( axis, angle );
		lastX = normalX;
		lastY = normalY;
		free( axis );
	}
	else if( mouseLeft == PRESSED ) {

		if( lastX == -1 ) {
			lastX = normalX;
			lastY = normalY;
			return;
		}

		// TRANSLATE THE CUBE IN THE XY PLANE

		diffX = (normalX - lastX)/scale;
		diffY = (normalY - lastY)/scale;

		mouseMoveXY( diffX, diffY );

		lastX = normalX;
		lastY = normalY;
		
	}

	display();
}

void mouseScroll( short zdelta ) {

		// Rotate the cube around an axis perpendicular to the screen

		double scale = WHEEL_DELTA*15.0; 

		double angle = getAngle( zdelta/scale, 0 );
		
		// If the mouse stroke was up-down instead of down-up,
		// rotate the cube in the opposite direction

		if( zdelta < 0 ) {
			angle = -angle;
		}

		Vector * axis = makeVector( 0.0, 0.0, 1.0 );
		rotateCube( axis, angle );
		free(axis);
		

}

CString rotAt( int frame ) {

	Frame * f = frames[ frame ];
	Rotation * rot = f->rotation;
	CString result;
	result.Format( "Rotate %f %f %f %f\n", (360.0/(2*3.14159))*rot->angle, rot->axis->x, rot->axis->y, rot->axis->z );
	return result;

}

float * rotArrayAt( int f ) {

	float * res = new float[4];
	res[0] = frames[f]->rotation->angle;
	res[1] = frames[f]->rotation->axis->x;
	res[2] = frames[f]->rotation->axis->y;
	res[3] = frames[f]->rotation->axis->z;
	return res;

}

float * transArrayAt( int f ) {

	float * res = new float[3];
	res[0] = frames[f]->translate->x;
	res[1] = frames[f]->translate->y;
	res[2] = frames[f]->translate->z;
	return res;

}

CString transAt( int frame ) {

	Frame * f = frames[ frame ];
	Vector * trans = f->translate;
	//Vector * cent = f->center;
	CString result;
	result.Format( "Translate %f %f %f\n", trans->x, trans->y, trans->z );
	//result.Format( "Translate %f %f %f\n", cent->x, cent->y, cent->z );
	return result;

}

void setRotAt( int frame, float deg, float x_ax, float y_ax, float z_ax ) {

	Frame * f = frames[frame];
	f->rotation->angle = deg;
	f->rotation->axis->x = x_ax;
	f->rotation->axis->y = y_ax;
	f->rotation->axis->z = z_ax;


}

void setTransAt( int frame, float x, float y, float z ) {

	Frame * f = frames[frame];
	f->translate->x = x;
	f->translate->y = y;
	f->translate->z = z;
	f->center->x = x + absoluteCenter->x;
	f->center->y = y + absoluteCenter->y;
	f->center->z = z + absoluteCenter->z;


}

void calculateFrame( int frame ) {

	Frame * currFrame = frames[frame];

	frontAndCenter();

	translate( makeVector( -center->x, -center->y, -center->z ) );

	rotateCube( currFrame->rotation->axis, currFrame->rotation->angle );
		
	for( int j = 0; j < 8; j++ ) {

		currFrame->vertices[j]->x = cubeVertices[j]->x + currFrame->center->x;
		currFrame->vertices[j]->y = cubeVertices[j]->y + currFrame->center->y;
		currFrame->vertices[j]->z = cubeVertices[j]->z + currFrame->center->z;

	} 

}

// Here's the main
void initDraw( int numFrames )
{
	
	// Define colors
	BLACK = makeColor( 0, 0, 0 );
	WHITE = makeColor( 1, 1, 1 );
	PINK = makeColor( .73, .56, .56);
	RED = makeColor( 1, 0, 0);
	LIGHT_GREEN = makeColor( .2, .8, .2);
	GREEN = makeColor( 0, 1, 0);
	LIGHT_BLUE = makeColor( .75, .85, .85);
	BLUE = makeColor( 0, 0, 1);
	GOLD = makeColor( .8, .5, .2);
	YELLOW = makeColor( 1, 1, 0);
	VIOLET = makeColor( .31, .18, .31);
	MAGENTA = makeColor( 1, 0, 1);
	SEA_GREEN = makeColor( .14, .55, .41);
	TURQUOISE = makeColor( .68, .91, .91);

	// define position of camera

	camera = makeVector( 0, 0, 0 );

	// Define intitial vertex positions of the cube
	cubeVertices[0] = makeVector( 1.0, 1.0, 3.0 );
	cubeVertices[1] = makeVector( -1.0, 1.0, 3.0 );
	cubeVertices[2] = makeVector( -1.0, -1.0, 3.0 );
	cubeVertices[3] = makeVector( 1.0, -1.0, 3.0 );
	cubeVertices[4] = makeVector( 1.0, 1.0, 5.0 );
	cubeVertices[5] = makeVector( -1.0, 1.0, 5.0 );
	cubeVertices[6] = makeVector( -1.0, -1.0, 5.0 );
	cubeVertices[7] = makeVector( 1.0, -1.0, 5.0 );
	
	center = makeVector( 0.0, 0.0, 4.0 );
	absoluteCenter = makeVector( 0.0, 0.0, 4.0 );

	frames = new Frame * [ numFrames ];
	makeFrames( 0, numFrames-1 );

	currRotation = makeRotation ( makeVector( 1.0, 0.0, 0.0), 0.0 );

	//rotateCube( makeVector( 0.0, 1.0, 0.0 ), 3.14159/4 );
   
}
