/*
 * A little program to figure out the single axis+angle that
 * corresponds to a series of rotations.
 *
 * Written by Gavin Bell.
 */

#include "stdafx.h"
#include <stdio.h>
#include <math.h>


#define MAX_ROTATIONS 10

/*
 * Convert axis/angle to quaternion form:
 * Assumes axis is unit-length, angle is in radians.
 */
void
to_quaternion(float axis[3], float angle, float result[4])
{
    int i;
    for (i = 0; i < 3; i++) {
	result[i] = axis[i]*sin(angle / 2.0);
    }
    result[3] = cos(angle / 2.0);
}
/*
 * Convert unit-length quaternion to axis/angle.
 */
void
from_quaternion(float q[4], float axis[3], float *angle)
{
    int i;
    *angle = acos(q[3]) * 2.0;
    for (i = 0; i < 3; i++) {
	axis[i] = q[i] / sin(*angle / 2.0);
    }
}

/*
 * Multiply two quaternions together.  Result may be the same as
 * either input:
 */
void
multiply_quaternion(float q1[4], float q2[4], float result[4])
{
    float tmp[4];
    tmp[0] = q2[3] * q1[0] + q2[0] * q1[3] +
	q2[1] * q1[2] - q2[2] * q1[1];
    tmp[1] = q2[3] * q1[1] + q2[1] * q1[3] +
	q2[2] * q1[0] - q2[0] * q1[2];
    tmp[2] = q2[3] * q1[2] + q2[2] * q1[3] +
	q2[0] * q1[1] - q2[1] * q1[0];
    tmp[3] = q2[3] * q1[3] - q2[0] * q1[0] -
	q2[1] * q1[1] - q2[2] * q1[2];
    result[0] = tmp[0]; result[1] = tmp[1];
    result[2] = tmp[2]; result[3] = tmp[3];
}

void perform( float axis[MAX_ROTATIONS][3], angle[MAXROTATIONS] ) {

	float q[4], result_axis[3], result_angle;

	to_quaternion(axis[0], angle[0], q);
    for (i = 1; i < num_entered; i++) {
		float q1[4];

		to_quaternion(axis[i], angle[i], q1);
		multiply_quaternion(q, q1, q);
    }
    from_quaternion(q, result_axis, &result_angle);


}
